<?php
/*
Plugin Name: DailyFratze Widget
Description: Adds a fratze to your sidebar
Author: Michael Simons
Version: 0.6b
Author URI: http://michael-simons.eu

Changelog
0.3 -> 0.4
	removed base64 encoding of image data as it is not supported in ie
0.4 -> 0.4a
	cached the image
0.4a -> 0.4b
	removed the port from the generated link
0.4b -> 0.5
	added the option to include a css class on the generated image tag
0.5 -> 0.5a
	new rss urls
0.5a -> 0.6
    Optionale Anzeige des aktuellsten Posts unterm Bild, eingebunden in ein Span mit der css Klasse
    fratzenpost
0.6 -> 0.6a
    Fehler im Caching der Bilder korrigiert
0.6a -> 0.6b
    Zeichensatz des Feedoutput (letzter Blogpost) dem des Blogs angepasst (huhu prego)
*/

function dailyfratze_latest_fratze() {
    dailyfratze_private_doit(get_option('widget_dailyfratze'));
}

function dailyfratze_charts($new_col = "#333333", $old_col = "#DDDDDD") {
	set_magic_quotes_runtime(0);

	$max_time = 4 * 60 * 60 ;

	$r_new = hexdec(substr($new_col, 1, 2));
	$r_old = hexdec(substr($old_col, 1, 2));
	$r_range = ($r_old-$r_new);

	$g_new = hexdec(substr($new_col, 3, 2));
	$g_old = hexdec(substr($old_col, 3, 2));
	$g_range = ($g_old-$g_new);

	$b_new = hexdec(substr($new_col, 5, 2));
	$b_old = hexdec(substr($old_col, 5, 2));
	$b_range = ($b_old-$b_new);

	$options = get_option('widget_dailyfratze');

	require_once(ABSPATH . WPINC . '/rss.php');
    define('MAGPIE_CACHE_ON',  false);
    define("MAGPIE_OUTPUT_ENCODING", "UTF-8");
    $fratzenschneider = $options['fratzenschneider'];
  	$feed = fetch_rss("http://dailyfratze.de/feeds/recentlyplayed/" . $fratzenschneider);

	foreach ($feed->items as $item) {
		$content = $item['atom_content'];

		$d = array();
		// MagPIE doesn't parse atom_content... lame ass thingy
		if (preg_match("/<track_title\s*>(.*)<\/track_title>\s*<track_artist\s*>(.*)<\/track_artist>\s*<track_album\s*>(.*)<\/track_album>\s*/u", $content, $d)) {
			$played = dailyfratze_private_parse_RFC3339($item['published']);
			$diff = time() - $played;
			$r = round($diff/$max_time*($r_range))+$r_new;
			$r = clamp(min($r_new, $r_old), max($r_new, $r_old), $r);
			$g = round($diff/$max_time*($g_range))+$g_new;
			$g = clamp(min($g_new, $g_old), max($g_new, $g_old), $g);
			$b = round($diff/$max_time*($b_range))+$b_new;
			$b = clamp(min($b_new, $b_old), max($b_new, $b_old), $b);
			$r_hex = str_pad(dechex($r), 2, '0', STR_PAD_LEFT);
			$g_hex = str_pad(dechex($g), 2, '0', STR_PAD_LEFT);
			$b_hex = str_pad(dechex($r), 2, '0', STR_PAD_LEFT);
			$colstr = " style=\"color: #".$r_hex.$g_hex.$b_hex.";\"";

			$title = "title=\"vom Album $d[3] zuletzt gespielt vor " . time_since($played) . '"';
			echo "<li $colstr $title><small>$d[2] - $d[1]</small></li>";
		}
	}
}

function widget_dailyfratze_init() {
	function widget_dailyfratze($args) {
		extract($args);

        $options = get_option('widget_dailyfratze');
        $title   = $options['title'];
        echo $before_widget . $before_title . $title . $after_title;
        dailyfratze_private_doit($options);
        echo $after_widget;
	}

	function widget_dailyfratze_control() {
		$options = get_option('widget_dailyfratze');
		if ( !is_array($options) )
			$options = array('title'=>'', 'fratzenschneider'=>'', 'size'=>'small', 'width'=>'');
		if ( $_POST['dailyfratze-submit'] ) {
			dailyfratze_private_retrive_option_input($options);
		}

		dailyfratze_private_echo_option_input($options);
	}

	register_sidebar_widget(array('DailyFratze.de', 'widgets'), 'widget_dailyfratze');
	register_widget_control(array('DailyFratze.de', 'widgets'), 'widget_dailyfratze_control', 300, 175);
}

function plugin_dailyfratze_options() {
	if (function_exists('add_options_page')) {
    	add_options_page('DailyFratze.de', 'DailyFratze.de', 0, basename(__FILE__), 'dailyfratze_private_options_subpanel');
	}
}

function dailyfratze_private_echo_option_input($options) {
    $title            = htmlspecialchars($options['title'], ENT_QUOTES);
	$fratzenschneider = htmlspecialchars($options['fratzenschneider'], ENT_QUOTES);
    $size             = htmlspecialchars($options['size'],  ENT_QUOTES);
    $width            = htmlspecialchars($options['width'], ENT_QUOTES);
    $css              = htmlspecialchars($options['css'],   ENT_QUOTES);
    
    $show_post        = isset($options['show_post']) && is_bool($options['show_post']) ? $options['show_post'] : false;
	$post_length      = isset($options['post_length']) && is_numeric($options['post_length']) ? $options['post_length'] : 30;
	$ellipsis         = htmlspecialchars(defined($options['ellipsis']) ? $options['ellipsis'] : '...');
        
	?>
	<table>
		<tr>
			<td><label for="dailyfratze-title">Titel</label></td>
			<td><input style="width: 200px;" id="dailyfratze-title" name="dailyfratze-title" type="text" title="Only used for the widget" value="<?php echo $title ; ?>" /></td>
		</tr>
		<tr>
			<td><label for="dailyfratze-fratzenschneider">Fratze</label></td>
			<td><input style="width: 200px;" id="dailyfratze-fratzenschneider" name="dailyfratze-fratzenschneider" type="text" value="<?php echo $fratzenschneider; ?>" /></td>
		</tr>
		<tr>
			<td><label for="dailyfratze-size"><?php echo __('Originalsize'); ?></label></td>
			<td>
				<select name="dailyfratze-size" id="dailyfratze-size">
					<option value="small" <?php echo $size == 'small' ? 'selected="selected"' : ''; ?>><?php echo __('Small'); ?></option>
					<option value="big"   <?php echo $size == 'big'   ? 'selected="selected"' : ''; ?>><?php echo __('Big'); ?></option>
	    	</select>
			</td>
		</tr>
		<tr>
			<td><label for="dailyfratze-width">Imagewidth</label></td>
			<td><input style="width: 200px;" id="dailyfratze-width" name="dailyfratze-width" type="text" value="<?php echo $width; ?>" /></td>
		</tr>
		<tr>
			<td><label for="dailyfratze-css">CSS Klasse fürs Bild</label></td>
			<td><input style="width: 200px;" id="dailyfratze-css" name="dailyfratze-css" type="text" value="<?php echo $css; ?>" /></td>
		</tr>
		<tr>
		    <td colspan="2">&#160;</td>
        </tr>		    
		<tr>
            <td><label>Tagebuch anzeigen</label></td>
            <td><input type="checkbox" id="dailyfratze-show_post" name="dailyfratze-show_post" <?php echo $show_post ? 'checked="checked"' : ''; ?>/></td>
         </tr>
         <tr>
            <td><label>Gesamtzahl Zeichen</label></td>
            <td><input style="width: 200px;" id="dailyfratze-ost_length" name="dailyfratze-post_length" type="text" value="<?php echo $post_length; ?>" /></td>            
         </tr>
          <tr>
            <td><label>Ellipsis</label></td>
            <td><input style="width: 200px;" id="dailyfratze-ellipsis" name="dailyfratze-ellipsis" type="text" value="<?php echo $ellipsis; ?>" /></td>            
         </tr>
	</table>
	<?php
	echo '<input type="hidden" id="dailyfratze-submit" name="dailyfratze-submit" value="1" />';
}

function dailyfratze_private_retrive_option_input($options) {
	$options['title']            = strip_tags(stripslashes($_POST['dailyfratze-title']));
	$options['fratzenschneider'] = strip_tags(stripslashes($_POST['dailyfratze-fratzenschneider']));
	$options['size']             = strip_tags(stripslashes($_POST['dailyfratze-size']));
	$options['width']            = strip_tags(stripslashes($_POST['dailyfratze-width']));
	$options['css']              = strip_tags(stripslashes($_POST['dailyfratze-css']));
    $options['show_post']        = isset($_POST['dailyfratze-show_post']);
    $options['post_length']      = strip_tags(stripslashes($_POST['dailyfratze-post_length']));
    $options['ellipsis']         = strip_tags(stripslashes($_POST['dailyfratze-ellipsis']));

	update_option('widget_dailyfratze', $options);
}

function dailyfratze_private_doit($options) {
    set_magic_quotes_runtime(0);

    require_once(ABSPATH . WPINC . '/rss.php');
    define('MAGPIE_CACHE_ON',  true);
    define("MAGPIE_OUTPUT_ENCODING", "UTF-8");
    $fratzenschneider = $options['fratzenschneider'];
	$width            = $options['width'];
	$css              = $options['css'];
	$show_post        = isset($options['show_post']) && is_bool($options['show_post']) ? $options['show_post'] : false;
	$post_length      = $options['post_length'] . ' || ' . defined($options['post_length']) && is_numeric($options['post_length']) ? $options['post_length'] : 30;
	$ellipsis         = isset($options['ellipsis']) ? $options['ellipsis'] : '...';
	
    $fratzen = fetch_rss("http://dailyfratze.de/feeds/fratzen/" . $fratzenschneider);    
    foreach ($fratzen->items as $item) {
        $m = null;
        preg_match("/(?P<y>(19|20)\d\d)\/(?P<m>(0?[1-9]|1[012]))\/(?P<d>[0-3]?\d)/", $item['id'], $m);
	    $pics[] = array(
	                'big'   => $item['link_image_big'],
	                'small' => $item['link_image_small'],
	                'title' => $item['title'],
			        'id'    => preg_replace('/:\d+\//', '/', substr($item['id'], 0, strrpos($item['id'], '/'))),
			        'd'     => mktime(0,0,0, $m['m'], $m['d'], $m['y']),
			        'u'     => strtotime($item['updated'])
	              );
    }
    $i = 0;

    $pout = '';
    if($show_post) {
        $pic_posts = array();
        $posts = fetch_rss("http://dailyfratze.de/feeds/posts/" . $fratzenschneider);
        foreach ($posts->items as $item) {
            $m = null;
            preg_match("/(?P<y>(19|20)\d\d)\/(?P<m>(0?[1-9]|1[012]))\/(?P<d>[0-3]?\d)/", $item['link'], $m);
            $d = mktime(0,0,0, $m['m'], $m['d'], $m['y']);
            if($d == $pics[$i]['d']) {
                $t = strtotime($item['published']);
                $pic_posts[$t][] = dailyfratze_private_trim(strip_tags($item['atom_content']));
                $pic_posts[$t][] = $item['link'];
            }
        }
        if(sizeof($pic_posts) > 0) {                            
            krsort ($pic_posts, SORT_NUMERIC);        
            $pic_post = reset($pic_posts);            
            $pout = $post_length <= 0 ? $pic_post[0] : substr($pic_post[0],0,$post_length-strlen($ellipsis));
            if(strlen($pout) < strlen($pic_post[0]))
                $pout .= '<a href="' . $pic_post[1] . '">' . $ellipsis . '</a>';        
            $pout = '<br /><span class="fratzenpost">' . $pout . '</span>';          
        }
    }           
    
    $fname = $pics[$i][$options['size'] == '' || $options['size'] == null ? 'small' : $options['size']];

	$cacheDirectory = ABSPATH . 'wp-content/cache/';
	$cacheFile      = 'dailyfratze.cache.jpg';
	$cacheTime      = 3600;

	$cachefile_created = @file_exists($cacheDirectory . $cacheFile) ? @filemtime($cacheDirectory . $cacheFile) : 0;
	if ((time() - $cacheTime > $cachefile_created) || $cachefile_created < $pics[$i]['u']) {
	   	$myFile = fopen($cacheDirectory . $cacheFile, "w");
		fwrite($myFile, file_get_contents($fname));
		fclose($myFile);
	}

    echo '<a href="' . $pics[$i]['id']. '">';
	echo '<img class="' . $css . '" src="' .get_option('siteurl') . '/wp-content/cache/' . $cacheFile . '" alt="'. $pics[$i]['title']  .'" title="' .$pics[$i]['title'] . '"' . ($width != '' ? 'width="' . $width . '" ' : '') .   '/>';
	echo '</a>';		
    $previous_encoding = iconv_get_encoding();    
    iconv_set_encoding("input_encoding",    "UTF-8");
    iconv_set_encoding("internal_encoding", "UTF-8");    
    iconv_set_encoding("output_encoding",   get_option('blog_charset'));    
    ob_start("ob_iconv_handler");
	print $pout;	
    ob_end_flush();    
    iconv_set_encoding("input_encoding",    $previous_encoding['input_encoding']);
    iconv_set_encoding("internal_encoding", $previous_encoding['internal_encoding']);            
    iconv_set_encoding("output_encoding",   $previous_encoding['output_encoding']);            
}

function dailyfratze_private_options_subpanel() {
    $options = get_option('widget_dailyfratze');
	if ( !is_array($options) )
		$options = array('title'=>'', 'fratzenschneider'=>'', 'size'=>'small', 'width'=>'');
    if (isset($_POST['info_update'])) {
        ?><div class="updated"><p><strong><?php
		dailyfratze_private_retrive_option_input(&$options);
        _e('Done.');
        ?></strong></p></div><?php
    } ?>

    <div class="wrap">
    <form method="post" action="">
        <h2>DailyFratze Options</h2>
        <fieldset id="set1">
        <?php dailyfratze_private_echo_option_input($options); ?>
        </fieldset>
        <div class="submit">
            <input type="submit" name="info_update" value="<?php
            _e('Update options')
            ?>" /></div>
    </form>
    </div><?php
}

function dailyfratze_private_parse_RFC3339($rfc3339) {
	$tmp = str_replace( "T", " ", $rfc3339); // Remove the T
	$tmp = ereg_replace("(\.[0-9]{1,})?", "", $tmp); // Remove the decimal and any numbers after it

	$datetime = substr($tmp, 0, 19);  // Grab the datetime part of the string
	$timezone = str_replace(":", "", substr($tmp, 19, 6)); // Grab the timezone, (-/+0000) PHP 4 doesn't support the colon

	return strtotime($datetime . " " . $timezone);
}

function dailyfratze_private_trim($str) {
    $pat[0] = "/^\s+/";
    $pat[1] = "/\s{2,}/";
    $pat[2] = "/\s+\$/";
    $rep[0] = "";
    $rep[1] = " ";
    $rep[2] = "";
    return preg_replace($pat,$rep,$str);
}

add_action('widgets_init', 'widget_dailyfratze_init');
add_action('admin_menu', 'plugin_dailyfratze_options');
?>